@@ -5,11 +5,10 @@
 #' @param name.list list of names
 #' @param data.list list of data
 #' @param initial.list initial values for analysis, Default: list()
-#' @param saved.steps define the number of iterations/steps/chains in the MCMC simulations, Default: 10000
-#' @param thinned.steps save every kth step of the original saved.steps, Default: 1
 #' @param run.contrasts logical, indicating whether or not to run contrasts, Default: FALSE
 #' @param use.contrast choose from "between", "within" and "mixed". Between compare groups at different conditions. Within compare a group at different conditions. Mixed compute all comparisons. 
 #' @param contrasts define contrasts to use for analysis (defaults to all) , Default: NULL
+#' @param run.ppp logical, indicating whether or not to conduct ppp analysis, Default: FALSE
 #' @param n.data sample size for each parameter
 #' @param credible.region summarize uncertainty by defining a region of most credible values (e.g., 95 percent of the distribution), Default: 0.95
 #' @param save.data logical, indicating whether or not to save data, Default: FALSE
@@ -38,7 +37,6 @@
 #'  \code{\link[parallel]{detectCores}}
 #'  \code{\link[coda]{as.mcmc.list}},\code{\link[coda]{varnames}}
 #'  \code{\link[plyr]{rbind.fill}}
-#'  \code{\link[robust]{covRob}}
 #'  \code{\link[stats]{cor}},\code{\link[stats]{cov}},\code{\link[stats]{sd}}
 #'  \code{\link[MASS]{mvrnorm}}
 #'  \code{\link[utils]{write.table}}
@@ -48,18 +46,17 @@
 #' @importFrom parallel detectCores
 #' @importFrom coda as.mcmc.list varnames
 #' @importFrom stats cor cov sd
-#' @importFrom MASS mvrnorm
+#' @importFrom MASS mvrnorm 
 #' @importFrom utils write.table
 RunMCMC <- function(jags.model, 
                     params, 
                     name.list,
                     data.list, 
                     initial.list,
-                    saved.steps, 
-                    thinned.steps,
                     run.contrasts,
                     use.contrast,
                     contrasts,
+                    run.ppp,
                     n.data,
                     credible.region, 
                     save.data = FALSE, 
@@ -84,26 +81,6 @@ RunMCMC <- function(jags.model,
                     ...
 ) {
   
-  # Number of saved steps
-  saved.steps <- saved.steps
-  # Number of thinned steps (keep only every kth step in chain)
-  thinned.steps <- thinned.steps
-  # Number of adapting steps
-  adapt.steps <- max ( (saved.steps * 5) / 100 , 2000 )
-  # Number of burn-in steps
-  burnin.steps <- max ( (saved.steps * 7.5) / 100 , 3000 )
-  
-  # Runjags options
-  # Disable redundant warnings
-  try( runjags::runjags.options( inits.warning=FALSE , rng.warning=FALSE ) )
-  # Set runjags method and number of chains
-  detect.cores <- parallel::detectCores()
-  jags.method <- if ( !is.finite(detect.cores || detect.cores < 4) ) "rJags" else "parallel"
-  n.chains <- if ( detect.cores >= 4 ) 4 else detect.cores
-  
-  # Number of samples
-  n.samples <- ceiling(saved.steps / n.chains)
-  
   # Name list
   project.name <- name.list$project.name
   project.data <- name.list$project.data
@@ -113,12 +90,40 @@ RunMCMC <- function(jags.model,
   job.title <- name.list$job.title
   job.names <- name.list$job.names
   job.group <- name.list$job.group
+  saved.steps = name.list$saved.steps
+  thinned.steps = name.list$thinned.steps
+  adapt.steps = name.list$adapt.steps
+  burnin.steps = name.list$burnin.steps
   jags.seed <- name.list$jags.seed
+  jags.method <- name.list$jags.method
+  jags.chains <- name.list$jags.chains
+  
+  # Add jags seed to initial list
   initial.list <- c(initial.list, .RNG.seed = jags.seed)
   
+  # Number of adapting steps
+  if (!length(adapt.steps)) adapt.steps <- max ( (saved.steps * 5) / 100 , 2000 )
+  # Number of burn-in steps
+  if (!length(burnin.steps)) burnin.steps <- max ( (saved.steps * 7.5) / 100 , 3000 )
+  
+  # Runjags options
+  # Disable redundant warnings
+  try( runjags::runjags.options( inits.warning=FALSE , rng.warning=FALSE ) )
+  # Set runjags method and number of chains
+  if (!length(jags.method)) {
+    detect.cores <- parallel::detectCores()
+    jags.method <- if ( !is.finite(detect.cores || detect.cores < 4) ) "simple" else "parallel"
+  }
+  if (!length(jags.chains)) {
+    jags.chains <- if ( detect.cores >= 4 ) 4 else detect.cores
+  }
+  
+  # Number of samples
+  n.samples <- ceiling(saved.steps / jags.chains)
+  
   # Display start time
   start.time <- Sys.time()
-  cat("\n",format(start.time, "Started at %d.%m.%Y - %H:%M:%S"),"\n")
+  cat(format(start.time,"\nStarted at %d.%m.%Y - %H:%M:%S\n"))
   
   # conduct JAGS
   if ( is.null(merge.MCMC) ) {
@@ -129,7 +134,7 @@ RunMCMC <- function(jags.model,
                                     monitor = params , 
                                     data = data.list ,  
                                     inits = initial.list , 
-                                    n.chains = n.chains ,
+                                    n.chains = jags.chains ,
                                     adapt = adapt.steps ,
                                     burnin = burnin.steps , 
                                     sample = n.samples ,
@@ -155,27 +160,95 @@ RunMCMC <- function(jags.model,
   colnames(matrix.MCMC) <- unlist(lapply(colnames(matrix.MCMC), function (x) if(regexpr('\\[', x)[1]<0) paste0(x,"[1]") else x ))
   coda::varnames(data.MCMC) <- unlist(lapply(coda::varnames(data.MCMC), function (x) if(regexpr('\\[', x)[1]<0) paste0(x,"[1]") else x ))
   
-  # Add sum to zero if predict metric/nominal
+  # Diagnostics
+  if (run.diag) {
+    cat("\nConducting diagnostics. Please wait.\n")
+    diag.start.time  <- Sys.time()
+    diag.length <- length(coda::varnames(data.MCMC))
+    diag.plots <- lapply(1:diag.length, function(i) { 
+      
+      x <- coda::varnames(data.MCMC)[i]    
+
+      if (stats::sd(matrix.MCMC[,x]) > 0) {
+        diag <- DiagMCMC(data.MCMC = data.MCMC, 
+                         par.name = x, 
+                         job.names = job.names, 
+                         job.group = job.group, 
+                         credible.region = credible.region
+        )
+      } else {
+        diag <- (paste0(x , " appears to be a constant. No available diagnostics."))
+      }
+      
+      ETA(diag.start.time , i , diag.length)
+      
+      return (diag)
+
+    } )
+
+    # Adding names to diagnostics 
+    names(diag.plots) <- coda::varnames(data.MCMC)
+    
+    if (save.data) {
+      cat("\nAdjusting and saving diagnostic plots. Please wait.\n")
+      
+      diag.plots <- diag.plots[lapply(diag.plots,length)>1]
+
+      #  Save plots as PowerPoint, Default is raster graphics. 
+      ## Change vector.graphic to TRUE if needed (not recommended) 
+      ParsePlot(diag.plots,
+                project.dir = paste0(project.dir,"Diagnostics/"),
+                project.name = project.name,
+                graphic.type = graphic.type, 
+                plot.size = plot.size,
+                save.data = save.data, 
+                vector.graphic = vector.graphic, 
+                point.size = point.size, 
+                font.type = font.type, 
+                one.file = one.file,
+                layout = layout,
+      )
+    }
+  }
+  
+  # Add contrasts for selected models
   if (run.contrasts) {
     
     q.levels <- data.list$q.levels
     
-    cat("\nCalculating contrasts, please wait (it may take some time).\n")
+    cat("\nComputing contrasts. Please wait.\n")
     
+    # Createsum to zero contrasts for mettric and nominal models
     if (model.type == "Metric" | model.type == "Nominal") {
-      matrix.MCMC <- cbind(matrix.MCMC,SumToZero(q.levels, matrix.MCMC, contrasts))
-    }
-    if (model.type == "Nominal") {
-      if (length(q.levels)>1) matrix.MCMC <- cbind( matrix.MCMC, MatrixCombn(matrix.MCMC,"o",q.levels,row.means=FALSE) )
-      contrast.type <- "b,o"
-    }
-    if (model.type == "Metric") {
-      if (length(q.levels)>1) matrix.MCMC <- cbind( matrix.MCMC, MatrixCombn(matrix.MCMC,"m,s",q.levels) )
-      contrast.type <- "b,m"
+      sum.zero <- SumToZero(q.levels, matrix.MCMC, contrasts)
+      matrix.MCMC <- cbind(matrix.MCMC, sum.zero)
     }
     
-    matrix.MCMC <- cbind( matrix.MCMC , 
-                          RunContrasts(contrast.type , q.levels , use.contrast , contrasts , matrix.MCMC , job.names) )
+    # Add odds (and odds-ratios/cohen's d) for nominal models
+    if (model.type == "Nominal") {
+        
+      contrast.type <- c("b","o")
+      # Create matrix combiantions for models with multiple factors
+      if (length(q.levels)>1) { 
+        odds <- MatrixCombn(matrix.MCMC, "o", q.levels, row.means=FALSE)
+        matrix.MCMC <- cbind(matrix.MCMC, odds)
+      }                                                                                        
+    }
+    # Add effect size cohen's d for metric model
+    if (model.type == "Metric") {
+      contrast.type <- c("b","m")
+      # Create matrix combiantions for models with multiple factors
+      if (length(q.levels)>1) {
+        mean.diff <- MatrixCombn(matrix.MCMC, "m,s",  q.levels)
+        matrix.MCMC <- cbind(matrix.MCMC, mean.diff)
+      }
+    }
+        
+    # Run contrasts and add contrasts to data matrix
+    done.contrasts <- do.call(cbind, lapply(contrast.type, function (x) {
+      RunContrasts(x , q.levels , use.contrast , contrasts , matrix.MCMC , job.names) 
+    }))
+    matrix.MCMC <- cbind(matrix.MCMC, done.contrasts)
   }
   
   # Add R^2 if regression
@@ -185,30 +258,40 @@ RunMCMC <- function(jags.model,
     n.x <- data.list$n.x
     y <- data.list$y
     q <- data.list$q
+    n <- data.list$n
     
-    r.squared <- lapply(1:q, function (i) {
+    r.squared <- do.call(cbind,lapply(1:q, function (i) {
       zbeta <- matrix.MCMC[,grep(paste0("^zbeta$|^zbeta\\[",i),colnames(matrix.MCMC))]
       
-      if (name.list$robust) {
-        if (!requireNamespace("robust", quietly = TRUE)) { 
-          warning("Package \"robust\" needed to run robust, frequntic covariate estimates.") 
-        }
-        cor.data <- robust::covRob( data.frame( x[,1:n.x[i]] , y ), 
-                                    corr = TRUE, 
-                                    ntrial = 50000)$cov[2]          
-      } else {
-        cor.data <- stats::cor( x[,1:n.x[i]] , y )
-      }      
+      # Compute R^2
+      cor.data <- stats::cor( x[,1:n.x[i]] , y  )  
       r.squared <- zbeta %*% matrix( cor.data , ncol=1 )
-      colnames(r.squared) <- sprintf("R^2 (block: %s)",i)
+      # Compute robust R^2
+      cor.data <- stats::cor( x[,1:n.x[i]] , y  , method="spearman")  
+      rob.r.squared <- zbeta %*% matrix( cor.data , ncol=1 )
+      # Compute adjusted R^2
+      adjusted.r <- as.matrix(1 - (1-r.squared) * ( (n-1) / (n -(q+1)) ))
+      # Compute adjusted robust R^2
+      adjusted.rob.r <- as.matrix(1 - (1-rob.r.squared) * ( (n-1) / (n -(q+1)) ))
       
-      return (r.squared)
-    })
-    matrix.MCMC <-  cbind(matrix.MCMC, do.call(cbind, r.squared))
-  }
+      # Add names
+      colnames(r.squared) <- sprintf("R^2 (block: %s)",i)
+      colnames(rob.r.squared) <- sprintf("Robust R^2 (block: %s)",i)
+      colnames(adjusted.r) <- sprintf("Adjusted R^2 (block: %s)",i)
+      colnames(adjusted.rob.r) <- sprintf("Adjusted Robust R^2 (block: %s)",i)
+
+      # Return as matrix
+      cbind(r.squared , rob.r.squared , adjusted.r , adjusted.rob.r)
+      
+    }))
+      
+    # Add R^2 to MCMC matrix  
+    matrix.MCMC <-  cbind(matrix.MCMC, r.squared)
+    
+  }  
   
   # Add PPP if SEM/CFA
-  if (model.type == "observed" & !name.list$robust) { 
+  if (run.ppp) { 
     y <- data.list$y
     lat <- data.list$lat
     factor.seq <- data.list$factor.seq
@@ -259,15 +342,8 @@ RunMCMC <- function(jags.model,
       if (pred.fit <= sim.fit) pppv <<- pppv + 1
       PPP <- as.numeric(pppv / i)     
       
-      eta <- Sys.time() + ( (nrow(matrix.MCMC) - i) * ( (Sys.time() - ppp.start.time) / i ) )
-      cat("\r" , 
-          sprintf("Progress: %.02f%% (PPP-value: %.03f). ETA: %s ", 
-                  (i * 100) / nrow(matrix.MCMC),
-                  PPP,
-                  format(eta,"%d.%m.%Y - %H:%M:%S")
-          )
-      )
-      
+      ETA(ppp.start.time , i , nrow(matrix.MCMC))
+
       # Create matrix with chi-square, discrepancy between predicted and simulated data and PPP
       PPP <-  as.data.frame(t(
         c("Fit (Predicted)" = pred.fit, 
@@ -280,110 +356,59 @@ RunMCMC <- function(jags.model,
       
     } )
     
-      if (requireNamespace("plyr", quietly = TRUE)) {
-        matrix.MCMC <-  cbind(matrix.MCMC, plyr::rbind.fill(PPP))
-      } else {
-        matrix.MCMC <-  cbind(matrix.MCMC, do.call(rbind,PPP))
-      }
+    if (requireNamespace("plyr", quietly = TRUE)) {
+      matrix.MCMC <-  cbind(matrix.MCMC, plyr::rbind.fill(PPP))
+    } else {
+      matrix.MCMC <-  cbind(matrix.MCMC, do.call(rbind,PPP))
+    }
   }
   
   # Find params from MCMC list
   params <- colnames(matrix.MCMC)
   
   # Create final posterior parameter indicies
+  summary.start.time <- Sys.time()
+  cat("\nSummarizing data of each parameter. Please wait.\n")
   summary.MCMC <- do.call(rbind,lapply(1:length(params), function(i) {
     
-    SumMCMC( par = matrix.MCMC[, params[i]] , 
-             par.names = params[i], 
-             job.names = job.names, 
-             job.group = job.group,
-             n.data = n.data,
-             credible.region = credible.region, 
-             ROPE = ROPE
-    )
-  } ) ) 
-  
-  # Diagnostics
-  if (run.diag) {
-    cat("\nConducting diagnostics, please wait (it may take some time).\n")
-    diag.start.time  <- Sys.time()
-    diag.length <- length(coda::varnames(data.MCMC))
-    diag.plots <- lapply(1:diag.length, function(i) { 
-      
-      x <- coda::varnames(data.MCMC)[i]    
-      
-      if (stats::sd(matrix.MCMC[,x]) == 0) {
-        cat("\n" , x , "appears to be a constant. Skipping diagnostics.\n")
-        return (NULL)
-      } else {
-        diag <- DiagMCMC(data.MCMC = data.MCMC, 
-                         par.name = x, 
-                         job.names = job.names, 
-                         job.group = job.group, 
-                         project.name = project.name, 
-                         credible.region = credible.region, 
-                         save.data = save.data, 
-                         project.dir = project.dir,
-                         monochrome = monochrome,
-                         plot.colors = plot.colors,
-                         font.type = font.type      
-        )
-        
-        eta <- Sys.time() + ( (diag.length - i) * ( (Sys.time() - diag.start.time) / i ) ) 
-        cat("\r", 
-            sprintf("Progress: %.02f%% (parameter: %s/%s). ETA: %s ", 
-                    (i * 100) / diag.length,
-                    i,
-                    diag.length,
-                    format(eta,"%d.%m.%Y - %H:%M:%S")
-            )
-        )
-      }
-      
-      return (diag)
-    } )
+  summary <- SumMCMC( par = matrix.MCMC[, params[i]] , 
+                      par.names = params[i], 
+                      job.names = job.names, 
+                      job.group = job.group,
+                      n.data = n.data,
+                      credible.region = credible.region, 
+                      ROPE = ROPE
+  )
     
-    if (save.data) {
-      cat("\nSaving diagnostics. Please wait.\n")
-      # Remove empty elements from list
-      diag.plots <- Filter(length, diag.plots)
+  ETA(summary.start.time , i , length(params))
+
+  return (summary)
       
-      #  Save plots as PowerPoint, Default is raster graphics. 
-      ## Change vector.graphic to TRUE if needed (not recommended) 
-      ParsePlot(diag.plots,
-                project.dir = paste0(project.dir,"Diagnostics/"),
-                project.name = project.name,
-                graphic.type = "pptx", 
-                plot.size = "15,10",
-                save.data = save.data, 
-                vector.graphic = vector.graphic, 
-                point.size = 15, 
-                font.type = "serif", 
-                one.file = TRUE,
-                layout = "pw",
-      )
-    }
-  }
-  
+  } ) ) 
+        
   # Display completion and running time
   stop.time <- Sys.time()
   total.time <- capture.output(difftime(stop.time, start.time))
-  cat("\n",format(stop.time, "Completed at %d.%m.%Y - %H:%M:%S"),
-      "\n",gsub("Time difference of","Running time:",total.time),"\n\n")
-  
+  cat(format(stop.time,"\nCompleted at %d.%m.%Y - %H:%M:%S\n"), 
+      gsub("Time difference of","Running time:",total.time),"\n", sep="")
+        
   # Create MCMC and summary list (without chain information)
   final.MCMC <- list( raw.MCMC = data.MCMC, 
                       matrix.MCMC = matrix.MCMC,
                       summary.MCMC = summary.MCMC, 
                       name.list = name.list,
+                      initial.list = initial.list, 
                       data.list = data.list, 
                       jags.model = jags.model,
                       run.time = c(start.time,stop.time)
   )
   
+  # If applicable, add diag plots
+  if (run.diag & !save.data) final.MCMC <- c(final.MCMC, diag = list(diag.plots))
   
-  # Save if requsted
+  # Save data if requested
   if (save.data) {  
+    cat("\nSaving data. Please wait.\n")
     # If requested, save summary as csv 
     data.file.name <- RemoveSpaces(paste0(project.dir,project.name,".csv"))
     utils::write.table(summary.MCMC, data.file.name, sep = sep)
@@ -397,6 +422,6 @@ RunMCMC <- function(jags.model,
                     data.file.name = data.file.name,
                     MCMC.file.name = MCMC.file.name)
   }  
-  
+      
   return (final.MCMC)
 }
